/*
 * Copyright (C) 2012 Department of General and Computational Linguistics,
 * University of Tuebingen
 *
 * This file is part of the Java Relatedness API to GermaNet.
 *
 * The Java Relatedness API to GermaNet is free software: you can redistribute
 * it and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * The Java Relatedness API to GermaNet is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this API; if not, see <http://www.gnu.org/licenses/>.
 */
package de.tuebingen.uni.sfs.germanet.relatedness.gui;

/**
 * @author University of Tuebingen, Department of Linguistics
 * (germanetinfo at sfs.uni-tuebingen.de)
 */
import de.tuebingen.uni.sfs.germanet.api.*;
import de.tuebingen.uni.sfs.germanet.relatedness.*;
import static de.tuebingen.uni.sfs.germanet.relatedness.gui.MainFrame.gnet;
import java.util.*;

public class RelatednessMeasureOutput {

    /**
     * 
     * @param s1 the list of synsets for Word 1
     * @param s2 the list of synsets for Word 2
     * @param measure the measure which was chosen to be calculated
     * @param gnet Germanet
     * @param frequencies 
     * @return The chosen Relatedness measures for all the synsets of Word 1 and Word 2
     * 
     */
    public static String resultAllSynsets(List<Synset> s1, List<Synset> s2, String measure, GermaNet gnet, HashMap<String, Long> frequencies) {
        String output = "";
        Relatedness rel = new Relatedness(gnet);
        //going through all the synsets for Word 1
        for (Synset synset1 : s1) {
            //going through all the synsets of Word 2
            for (Synset synset2 : s2) {
                
                output += "Measures for synset with ID " + synset1.getId()+": "+synset1.getAllOrthForms() + " and synset with ID " + synset2.getId()+": "+synset2.getAllOrthForms() + ":\n\n";
                //if Path Measure or Alle Measures is chosen
                if (measure.equalsIgnoreCase("path") || measure.equalsIgnoreCase("All Measures")) {
                    
                    output += "Path: " + rel.path(synset1, synset2).getResult() + "\n";
                }
                if (measure.equalsIgnoreCase("Hirst and StOnge") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Hirst and StOnge: " + rel.hirstAndStOnge(synset1, synset2).getResult() + "\n";
                }

                if (measure.equalsIgnoreCase("Jiang and Conrath") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Jiang And Conrath: " + rel.jiangAndConrath(synset1, synset2, frequencies).getResult()+ "\n";
                }
                if (measure.equalsIgnoreCase("Leacock and Chodorow") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Leacock and Chodorow: " + rel.leacockAndChodorow(synset1, synset2).getResult() + "\n";
                }

                if (measure.equalsIgnoreCase("Resnik") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Resnik: " + rel.resnik(synset1, synset2, frequencies).getResult()+" Normalized Result:"+rel.resnik(synset1,synset2,frequencies).getNormalizedResult() + " \n";
                }
                
               
                //problems initianizing stemmer for lesk
                if(measure.equalsIgnoreCase("Lesk")||measure.equalsIgnoreCase("All Measures")){
                    output+= "Lesk: "+rel.lesk(synset1,synset2,gnet).getResult()+"\n";
                }
                if (measure.equalsIgnoreCase("Wu and Palmer") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Wu and Palmer: " + rel.wuAndPalmer(synset1, synset2).getResult() +" "+ "\n";
                }                
                if (measure.equalsIgnoreCase("Lin") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Lin: " + rel.lin(synset1, synset2, frequencies) + "\n";
                }
                output+="__________________________________________________________________\n\n";
            }
          
        }
        return output;
    }
    
    /**
     * 
     * @param s1 the list of synsets for Word 1
     * @param s2 the list of synsets for Word 2
     * @param measure the measure which was chosen to be calculated
     * @param gnet Germanet
     * @param frequencies 
     * @return The chosen Relatedness measures for all the synsets of Word 1 and Word 2 (formated for CSV output)
     * 
     */
    public static String resultAllSynsetsCSV(String w1, String w2, List<Synset> s1, List<Synset> s2, String measure, GermaNet gnet, HashMap<String, Long> frequencies, String csvDltr) {
        //String output = "\"Word #1\", \"Word #2\", \"Synset #1\", \"Synset #2\", \"Path\", \"Hirst and StOnge\", \"Jiang And Conrath\", \"Leacock and Chodorow\", \"Resnik\", \"Resnik(normalized)\", \"Lesk\", \"Wu and Palmer\", \"Lin\"\n";
        String output = "";
        
        Relatedness rel = new Relatedness(gnet);
        //going through all the synsets for Word 1
        for (Synset synset1 : s1) {
            //going through all the synsets of Word 2
            for (Synset synset2 : s2) {
                // new line of results
                //output += "\"" + w1 + "\"" + csvDltr + " \"" + w2 + "\"" + csvDltr + " ";
                List<String> outputLine = new ArrayList<String>();
                outputLine.add(w1);
                outputLine.add(w2);
                
                //output += "\"" + synset1.getId() + ": " + synset1.getAllOrthForms() + "\"" + csvDltr + " \"" + synset2.getId() + ": " + synset2.getAllOrthForms() + "\"" + csvDltr + " ";
                outputLine.add(synset1.getId() + ": " + synset1.getAllOrthForms());
                outputLine.add(synset2.getId() + ": " + synset2.getAllOrthForms());
                //if Path Measure or Alle Measures is chosen
                if (measure.equalsIgnoreCase("path") || measure.equalsIgnoreCase("All Measures")) {                    
                    //output += "\"" + rel.path(synset1, synset2).getResult() + "\"" + csvDltr;
                    outputLine.add(String.valueOf(rel.path(synset1, synset2).getResult()));
                }
                else {
                    //output += "\"\"" + csvDltr + " ";
                    outputLine.add("");
                }
                if (measure.equalsIgnoreCase("Hirst and StOnge") || measure.equalsIgnoreCase("All Measures")) {
                    //output += "\"" + rel.hirstAndStOnge(synset1, synset2).getResult() + "\"" + csvDltr + " ";
                    outputLine.add(String.valueOf(rel.hirstAndStOnge(synset1, synset2).getResult()));
                }
                else {
                    //output += "\"\"" + csvDltr + " ";
                    outputLine.add("");
                }
                if (measure.equalsIgnoreCase("Jiang and Conrath") || measure.equalsIgnoreCase("All Measures")) {
                    //output += "\"" + rel.jiangAndConrath(synset1, synset2, frequencies).getResult() + "\"" + csvDltr + " ";
                    if (frequencies != null) {
                        Long fr1 = frequencies.get(synset1.getId()+"");
                        Long fr2 = frequencies.get(synset2.getId()+"");
                        if ((fr1!=null) && (fr2!=null)) {
                            if ((fr1>0) && (fr2>0)) {
                                outputLine.add(String.valueOf(rel.jiangAndConrath(synset1, synset2, frequencies).getResult()));
                            }
                            else {
                                outputLine.add("Frequency value equals zero");
                            }                                                                                    
                        }
                        else {
                            outputLine.add("Frequency value is not found");
                        }
                    }
                    else {
                        outputLine.add("Frequency list is not found");
                    }                    
                }
                else {
                    //output += "\"\"" + csvDltr + " ";
                    outputLine.add("");
                }
                if (measure.equalsIgnoreCase("Leacock and Chodorow") || measure.equalsIgnoreCase("All Measures")) {
                    //output += "\"" + rel.leacockAndChodorow(synset1, synset2).getResult() + "\"" + csvDltr + " ";
                    outputLine.add(String.valueOf(rel.leacockAndChodorow(synset1, synset2).getResult()));
                }
                else {
                    //output += "\"\"" + csvDltr + " ";
                    outputLine.add("");
                }
                if (measure.equalsIgnoreCase("Resnik") || measure.equalsIgnoreCase("All Measures")) {
                    //output += "\"" + rel.resnik(synset1, synset2, frequencies).getResult()+"\"" + csvDltr + " \"" + rel.resnik(synset1,synset2,frequencies).getNormalizedResult() + "\"" + csvDltr + " ";                    
                    if (frequencies != null) {
                        Long fr1 = frequencies.get(synset1.getId()+"");
                        Long fr2 = frequencies.get(synset2.getId()+"");
                        if ((fr1!=null) && (fr2!=null)) {
                            if ((fr1>0) && (fr2>0)) {
                                //outputLine.add(String.valueOf(rel.jiangAndConrath(synset1, synset2, frequencies).getResult()));
                                outputLine.add(String.valueOf(rel.resnik(synset1, synset2, frequencies).getResult()));
                                outputLine.add(String.valueOf(rel.resnik(synset1,synset2,frequencies).getNormalizedResult()));
                            }
                            else {
                                outputLine.add("Frequency value equals zero");
                                outputLine.add("");
                            }                                                                                    
                        }
                        else {
                            outputLine.add("Frequency value is not found");
                            outputLine.add("");
                        }
                    }
                    else {
                        outputLine.add("Frequency list is not found");
                        outputLine.add("");
                    }                                                       
                }
                else {
                    //output += "\"\"" + csvDltr + " \"\"" + csvDltr + " ";
                    outputLine.add("");
                    outputLine.add("");
                }
                               
                //problems initianizing stemmer for lesk
                if(measure.equalsIgnoreCase("Lesk")||measure.equalsIgnoreCase("All Measures")){
                    //output += "\"" + rel.lesk(synset1,synset2,gnet).getResult() + "\"" + csvDltr + " ";
                    outputLine.add(String.valueOf(rel.lesk(synset1,synset2,gnet).getResult()));
                }
                else {
                    //output += "\"\"" + csvDltr + " ";
                    outputLine.add("");
                }
                if (measure.equalsIgnoreCase("Wu and Palmer") || measure.equalsIgnoreCase("All Measures")) {
                    //output += "\"" + rel.wuAndPalmer(synset1, synset2).getResult() + "\"" + csvDltr + " ";
                    outputLine.add(String.valueOf(rel.wuAndPalmer(synset1, synset2).getResult()));
                }
                else {
                    //output += "\"\"" + csvDltr + " ";
                    outputLine.add("");
                }
                if (measure.equalsIgnoreCase("Lin") || measure.equalsIgnoreCase("All Measures")) {
                    //output += "\"" + rel.lin(synset1, synset2, frequencies) + "\"\n";
                    if (frequencies != null) {
                        Long fr1 = frequencies.get(synset1.getId()+"");
                        Long fr2 = frequencies.get(synset2.getId()+"");
                        if ((fr1!=null) && (fr2!=null)) {
                            if ((fr1>0) && (fr2>0)) {
                                outputLine.add(String.valueOf(rel.lin(synset1, synset2, frequencies)));
                            }
                            else {
                                outputLine.add("Frequency value equals zero");                                
                            }                                                                                    
                        }
                        else {
                            outputLine.add("Frequency value is not found");                            
                        }
                    }
                    else {
                        outputLine.add("Frequency list is not found");                        
                    }                                   
                }
                else {
                    //output += "\"\"\n";
                    outputLine.add("");
                }
                output += joinListList(outputLine, csvDltr) + "\n";
                // end of line
            }          
        }
        
        return output;
    }
    public static String joinListList(List<String> inputList, String separator) {
        String joinedString = "";
        for (int i = 0; i < inputList.size(); i++) {
            
            if ((inputList.get(i).contains(";")) || (inputList.get(i).contains(",")) || (inputList.get(i).contains("\t"))) {
                joinedString += "\"" + inputList.get(i) + "\"";               
            }
            else {
                joinedString += inputList.get(i);
            }
            if (i != (inputList.size()-1)) {
                joinedString += separator;
            }
        }
        return joinedString;
    }
    
    public static String result(Synset synset1,Synset synset2,String measure,GermaNet gnet, HashMap<String,Long>frequencies){
        String output="";
        Relatedness rel = new Relatedness(gnet);
         output += "Measures for" + synset1.toString() + "and " + synset2.toString() + ":\n";
                //if Path Measure or Alle Measures is chosen
              
                if (measure.equalsIgnoreCase("path") || measure.equalsIgnoreCase("All Measures")) {
                    
                    output += "Path: " + rel.path(synset1, synset2).getResult() + "\n";
                }
                if (measure.equalsIgnoreCase("Hirst and StOnge") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Hirst and StOnge: " + rel.wuAndPalmer(synset1, synset2).getResult() + "\n";
                }

                if (measure.equalsIgnoreCase("Jiang and Conrath") || measure.equalsIgnoreCase("All Measures")) {
                    if (frequencies != null) {
                        Long fr1 = frequencies.get(synset1.getId()+"");
                        Long fr2 = frequencies.get(synset2.getId()+"");
                        if ((fr1!=null) && (fr2!=null)) {
                            if ((fr1>0) && (fr2>0)) {
                                output += "Jiang And Conrath: " + rel.jiangAndConrath(synset1, synset2, frequencies) + "\n";
                            }
                            else {
                                 output += "Jiang And Conrath: Frequency value equals zero";
                            }                                                                                    
                        }
                        else {
                            output += "Jiang And Conrath: Frequency value is not found";
                        }
                    }
                    else {
                        output += "Jiang And Conrath: Frequency list is not found";
                    }             
                    
                }
                if (measure.equalsIgnoreCase("Leacock and Chodorow") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Leacock and Chodorow: " + rel.leacockAndChodorow(synset1, synset2) + "\n";
                }

                if (measure.equalsIgnoreCase("Resnik") || measure.equalsIgnoreCase("All Measures")) {
                    if (frequencies != null) {
                        Long fr1 = frequencies.get(synset1.getId()+"");
                        Long fr2 = frequencies.get(synset2.getId()+"");
                        if ((fr1!=null) && (fr2!=null)) {
                            if ((fr1>0) && (fr2>0)) {                               
                                output += "Resnik: " + rel.resnik(synset1, synset2, frequencies) + "\n";
                            }
                            else {
                                 output += "Resnik: Frequency value equals zero";
                            }                                                                                    
                        }
                        else {
                            output += "Resnik: Frequency value is not found";
                        }
                    }
                    else {
                        output += "Resnik: Frequency list is not found";
                    }                                                
                }
                
                 if(measure.equalsIgnoreCase("Lesk")||measure.equalsIgnoreCase("All Measures")){
                 output+= "Lesk: "+rel.lesk(synset1,synset2,gnet).getResult()+"\n";
                 }
                if (measure.equalsIgnoreCase("Wu and Palmer") || measure.equalsIgnoreCase("All Measures")) {
                    output += "Wu and Palmer: " + rel.wuAndPalmer(synset1, synset2).toString() + "\n";
                }
                
                if (measure.equalsIgnoreCase("Lin") || measure.equalsIgnoreCase("All Measures")) {
                    if (frequencies != null) {
                        Long fr1 = frequencies.get(synset1.getId()+"");
                        Long fr2 = frequencies.get(synset2.getId()+"");
                        if ((fr1!=null) && (fr2!=null)) {
                            if ((fr1>0) && (fr2>0)) {                                                               
                                output += "Lin: " + rel.lin(synset1, synset2, frequencies) + "\n";
                            }
                            else {
                                 output += "Lin: Frequency value equals zero";
                            }                                                                                    
                        }
                        else {
                            output += "Lin: Frequency value is not found";
                        }
                    }
                    else {
                        output += "Lin: Frequency list is not found";
                    }                            
                }
        
        return output;
        
    }
    
    public static String resultforWordList(List<List<String>> wordList,String measure,GermaNet gnet, HashMap<String,Long>frequencies, Integer posTagIndex, Integer csvDelIndex){
        String csvDltr = ";";
        if (csvDelIndex == 0) {
            csvDltr = ";";
        }
        if (csvDelIndex == 1) {
            csvDltr = ",";
        }
        if (csvDelIndex == 2) {
            csvDltr = "\t";
        }
        
        String output = "Word #1" + csvDltr + "Word #2" + csvDltr + "Synset #1" + csvDltr + "Synset #2" + csvDltr + "Path" + csvDltr + "Hirst and StOnge" + csvDltr + "Jiang And Conrath" + csvDltr + "Leacock and Chodorow" + csvDltr + "Resnik" + csvDltr + "Resnik(normalized)" + csvDltr + "Lesk" + csvDltr + "Wu and Palmer" + csvDltr + "Lin\n";
        //String output="";
        //System.out.println("POS = " + posTagIndex);
        String delimiters = "\\s+|\\,|\\;|\\t+";
        //for(String[] currLine:wordList){
        for (int wl = 0; wl < wordList.size(); wl++) { 
            List<String> currentPair = new ArrayList<String>();
            currentPair = wordList.get(wl);
            //output+="\n" + "Output for the words:\n";
            // This part of code was not working properly: the programm  could not split words within one line
            // I have added a regular expression that does it nicely
            //String[] currWords = currLine[0].split(delimiters);  
            //String[] currWords = ;  
        //System.out.println(wordList);
        //System.out.println(wordList.size());
            if(currentPair.size()==2){
                //String word1=currWords[0];
                //String word2=currWords[1];
                String word1 = currentPair.get(0);
                String word2 = currentPair.get(1);                                
                List<Synset> allSynsets1=gnet.getSynsets(word1);
                List<Synset> synsets1 = new ArrayList<Synset>();
                // Filtering by POS               
                for (int c1 = 0; c1 < allSynsets1.size(); c1++) {                    
                    WordCategory posTag1 = allSynsets1.get(c1).getWordCategory();
                    if (posTagIndex == 0) { // all parts of speech
                        synsets1.add(allSynsets1.get(c1));
                    }
                    if (posTagIndex == 1) { // nouns
                        if (posTag1.equals(allSynsets1.get(c1).getWordCategory().nomen)) {                        
                            synsets1.add(allSynsets1.get(c1));
                        }
                    }
                    if (posTagIndex == 2) { // adjectives
                        if (posTag1.equals(allSynsets1.get(c1).getWordCategory().adj)) {                        
                            synsets1.add(allSynsets1.get(c1));
                        }
                    }
                    if (posTagIndex == 3) { // verbs
                        if (posTag1.equals(allSynsets1.get(c1).getWordCategory().verben)) {                        
                            synsets1.add(allSynsets1.get(c1));
                        }
                    }
                }               
                                
                List<Synset> allSynsets2=gnet.getSynsets(word2);
                List<Synset> synsets2 = new ArrayList<Synset>();
                // Filtering by POS               
                for (int c2 = 0; c2 < allSynsets2.size(); c2++) {                    
                    WordCategory posTag2 = allSynsets2.get(c2).getWordCategory();
                    if (posTagIndex == 0) { // all parts of speech
                        synsets2.add(allSynsets2.get(c2));
                    }
                    if (posTagIndex == 1) { // nouns
                        if (posTag2.equals(allSynsets2.get(c2).getWordCategory().nomen)) {                        
                            synsets2.add(allSynsets2.get(c2));
                        }
                    }
                    if (posTagIndex == 2) { // adjectives
                        if (posTag2.equals(allSynsets2.get(c2).getWordCategory().adj)) {                        
                            synsets2.add(allSynsets2.get(c2));
                        }
                    }
                    if (posTagIndex == 3) { // verbs
                        if (posTag2.equals(allSynsets2.get(c2).getWordCategory().verben)) {                        
                            synsets2.add(allSynsets2.get(c2));
                        }
                    }
                }                                                                               
                if ((synsets1.isEmpty() == false) && (synsets2.isEmpty() == false)){                  
                    output += resultAllSynsetsCSV(word1, word2, synsets1, synsets2,measure,gnet,frequencies, csvDltr);
                }
                
                
                if ((synsets1.isEmpty() == true) || (synsets2.isEmpty() == true)){
                    List<String> outputWordLine = new ArrayList<String>();
                    if (allSynsets1.size() == 0) {
                        outputWordLine.add(word1 + " was not found in GermaNet");
                    }
                    else {
                        outputWordLine.add(word1);
                    }
                    if (allSynsets2.size() == 0) {
                        outputWordLine.add(word2 + " was not found in GermaNet");
                    }
                    else {
                        outputWordLine.add(word2);
                    }
                    for (int el = 0; el < 11; el++) {
                        outputWordLine.add(" - ");                        
                    }
                    if ((synsets1.isEmpty() == true) && (allSynsets1.size() > 0) && (synsets2.isEmpty() == true) && (allSynsets2.size() > 0)) {
                        output += "";
                    }
                    else {
                        output += joinListList(outputWordLine, csvDltr) + "\n";
                    }
                }
            }
            /*else{
                int i=0;
                while(i<currWords.length){
                    output+=currWords[i]+" ";
                    output+="\n Less or more than two words in line\n";
                    i++;
                }
            }*/
            //output+="\n";                        
        }
        return output;
    }
}
